/*

Copyright (C) 2009 NTT DATA INTELLILINK CORPORATION

This program is free software; you can redistribute it and/or
Modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation, version 2.

This program is distributed in the hope that it will be
useful, but WITHOUT ANY WARRANTY; without even the implied 
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
PURPOSE.  See the GNU General Public License for more details.

 */

package jp.co.intellilink.hinemos.check.conf;

import java.util.ArrayList;
import java.util.Iterator;

import jp.co.intellilink.hinemos.util.EjbConnectionManager;
import jp.co.intellilink.hinemos.util.Messages;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.clustercontrol.monitor.run.bean.MonitorInfo;
import com.clustercontrol.monitor.run.bean.MonitorTypeConstant;
import com.clustercontrol.snmp.ejb.session.MonitorSnmpController;

/**
 * SNMP監視情報を取得するクラス<br>
 * 
 * @version 1.0.0
 * @since 1.0.0
 */
public class CheckMonitorSnmp extends CheckMonitorString {

	// ログ出力
	private static Log log = LogFactory.getLog(CheckMonitorSnmp.class);

	/**
	 * SNMP監視の監視項目数、ノード数、通知数を取得する。<br>
	 */
	@SuppressWarnings("unchecked")
	public ArrayList<String> checkSnmp() {
		log.info(Messages.getMsg("Snmp.Start", new String[]{""}));

		//監視項目リスト取得
		ArrayList idList = getSnmpList();
		setIdList(idList);

		if(idList instanceof ArrayList){
			Iterator itr = idList.iterator();
			while(itr.hasNext()){
				ArrayList line = (ArrayList)itr.next();

				Object[] args = {(String)line.get(0)};
				log.info(Messages.getMsg("MonitorID", args));

				int monitorType = MonitorTypeConstant.stringToType((String)line.get(3));
				MonitorInfo info = getSnmp((String)line.get(0), monitorType);

				int nodeCount = getNodeCount(info);
				int notifyCount = 0;

				if(monitorType == MonitorTypeConstant.TYPE_NUMERIC){
					notifyCount = getNotifyCount(info);
				}
				else{
					if(getNotifyCount(info) > getJudgementNotifyCount(info))
						notifyCount = getNotifyCount(info);
					else
						notifyCount = getJudgementNotifyCount(info);
				}
				addNodeTotalCount(nodeCount);
				addNotifyTotalCount(nodeCount * notifyCount);
			}
		}

		log.info(Messages.getMsg("Snmp.End", new String[]{""}));

		return createOutput("Snmp");
	}

	/**
	 * SNMP監視情報を取得する。<br>
	 * 
	 * @param monitorId 取得対象の監視項目ID
	 * @return SNMP監視情報
	 */
	protected MonitorInfo getSnmp(String monitorId, int monitorType) {

		MonitorSnmpController snmp = EjbConnectionManager.getConnectionManager().getMonitorSnmpController();

		MonitorInfo info = null;
		try {
			info = snmp.getSnmp(monitorId, monitorType);
		} catch (Exception e) {
			log.error(Messages.getMsg("ConnectManagerFailed"), e);
			System.exit(14);
		}
		return info;
	}

	/**
	 * 監視項目リストを取得する。<br>
	 * 
	 * @return 監視項目一覧
	 */
	@SuppressWarnings("unchecked")
	protected ArrayList getSnmpList() {

		MonitorSnmpController snmp = EjbConnectionManager.getConnectionManager().getMonitorSnmpController();

		ArrayList records = null;
		try {
			records = snmp.getSnmpList();
		} catch (Exception e) {
			log.error(Messages.getMsg("ConnectManagerFailed"), e);
			System.exit(14);
		}
		return records;
	}
}